@@ -88,22 +88,18 @@ Dav1dRef *dav1d_ref_wrap(const uint8_t *const ptr,
     return res;
 }
 
-void dav1d_ref_inc(Dav1dRef *const ref) {
-    atomic_fetch_add(&ref->ref_cnt, 1);
-}
-
 void dav1d_ref_dec(Dav1dRef **const pref) {
     assert(pref != NULL);
 
     Dav1dRef *const ref = *pref;
     if (!ref) return;
 
+    *pref = NULL;
     if (atomic_fetch_sub(&ref->ref_cnt, 1) == 1) {
         const int free_ref = ref->free_ref;
         ref->free_callback(ref->const_data, ref->user_data);
         if (free_ref) free(ref);
     }
-    *pref = NULL;
 }
 
 int dav1d_ref_is_writable(Dav1dRef *const ref) {
